@@ -43,23 +43,21 @@ ellipse <- function(x,y,covmat,alpha=0.05,n=50){
 #' a third variable as a colour scale.
 #'
 #' @param xy matrix with columns \code{X, sX, Y, sY(, rXY)}
-#' @param xlim (optional) two-element vector with the x-axis limits
-#' @param ylim (optional) two-element vector with the y-axis limits
 #' @param alpha the probability cutoff for the error ellipses
 #' @param show.numbers logical flag (\code{TRUE} to show grain
 #'     numbers)
 #' @param show.ellipses show the data as:
 #' 
-#' \code{1}: points
+#' \code{0}: points
 #' 
-#' \code{2}: error ellipses
+#' \code{1}: error ellipses
 #' 
-#' \code{3}: error crosses
+#' \code{2}: error crosses
 #' 
 #' @param levels a vector with additional values to be displayed as
 #'     different background colours within the error ellipses.
 #' @param clabel label for the colour scale
-#' @param ellipse.col
+#' @param ellipse.fill
 #' Fill colour for the error ellipses. This can either be a single
 #' colour or multiple colours to form a colour ramp. Examples:
 #'
@@ -77,6 +75,9 @@ ellipse <- function(x,y,covmat,alpha=0.05,n=50){
 #' etc.
 #'
 #' For empty ellipses, set \code{ellipse.col=NA}
+#' @param ellipse.stroke the stroke colour for the error
+#'     ellipses. Follows the same formatting guidelines as
+#'     \code{ellipse.fill}
 #' @param fit the output of \code{york()} (optional).
 #' @param add if \code{TRUE}, adds the points and lines to the
 #'     existing plot.
@@ -90,13 +91,21 @@ ellipse <- function(x,y,covmat,alpha=0.05,n=50){
 #'     from the plot.
 #' @param omit vector with indices of aliquots that should be plotted
 #'     but omitted from the isochron age calculation.
-#' @param omit.col colour that should be used for the omitted
+#' @param omit.fill fill colour that should be used for the omitted
 #'     aliquots.
+#' @param omit.stroke stroke colour that should be used for the
+#'     omitted aliquots.
 #' @param addcolourbar add a colour bar to display the colours used to
 #'     \code{levels}
 #' @param bg background colour for the plot symbols (only used if
 #'     \code{show.ellipses=0}).
 #' @param cex plot symbol magnification.
+#' @param xlim (optional) two-element vector with the x-axis limits
+#' @param ylim (optional) two-element vector with the y-axis limits
+#' @param xlab (optional) x-axis label (only used when
+#'     \code{add=FALSE})
+#' @param ylab (optional) y-axis label (only used when
+#'     \code{add=FALSE})
 #' @param ... optional arguments to format the points and text.
 #' 
 #' @examples
@@ -109,38 +118,42 @@ ellipse <- function(x,y,covmat,alpha=0.05,n=50){
 #' dat <- cbind(X,sX,Y,sY)
 #' scatterplot(dat,fit=york(dat),show.ellipses=2)
 #' @export
-scatterplot <- function(xy,xlim=NA,ylim=NA,alpha=0.05,
-                        show.numbers=FALSE,show.ellipses=1,levels=NA,
-                        clabel="",ellipse.col=c("#00FF0080","#FF000080"),
-                        fit='none',add=FALSE,empty=FALSE,
-                        ci.col='gray80',line.col='black',lwd=1,
-                        hide=NULL,omit=NULL,omit.col=NA,
-                        addcolourbar=TRUE,bg,cex,...){
+scatterplot <- function(xy,alpha=0.05,show.numbers=FALSE,
+                        show.ellipses=1,levels=NA,clabel="",
+                        ellipse.fill=c("#00FF0080","#FF000080"),
+                        ellipse.stroke="black",fit='none',add=FALSE,
+                        empty=FALSE, ci.col='gray80',line.col='black',
+                        lwd=1,hide=NULL,omit=NULL,omit.fill=NA,
+                        omit.stroke="grey",addcolourbar=TRUE,
+                        bg,cex,xlim,ylim,xlab,ylab,...){
     ns <- nrow(xy)
     if (ncol(xy)==4) xy <- cbind(xy,rep(0,ns))
     sn <- 1:ns
     plotit <- sn%ni%hide
     calcit <- sn%ni%c(hide,omit)
     colnames(xy) <- c('X','sX','Y','sY','rXY')
-    if (any(is.na(xlim))) xlim <- get.limits(xy[plotit,'X'],xy[plotit,'sX'])
-    if (any(is.na(ylim))) ylim <- get.limits(xy[plotit,'Y'],xy[plotit,'sY'])
-    if (!add) graphics::plot(xlim,ylim,type='n',xlab='',ylab='',bty='n')
-    if (!add & empty) return()
-    if (!identical(fit,'none'))
+    if (missing(xlim)) xlim <- get.limits(xy[plotit,'X'],xy[plotit,'sX'])
+    if (missing(ylim)) ylim <- get.limits(xy[plotit,'Y'],xy[plotit,'sY'])
+    if (!add){
+        if (missing(xlab)) xlab <- ''
+        if (missing(ylab)) ylab <- ''
+        graphics::plot(xlim,ylim,type='n',xlab=xlab,ylab=ylab,bty='n')
+        if (empty) return()
+    }
+    if (!identical(fit,'none')){
         plot_isochron_line(fit,x=seq(xlim[1],xlim[2],length.out=100),
                            ci.col=ci.col,col=line.col,lwd=lwd)
+    }
     graphics::box()
     nolevels <- all(is.na(levels))
-    if (show.ellipses==2 && nolevels){
-        colour <- rep('black',ns)
-    } else {
-        colour <- set.ellipse.colours(ns=ns,levels=levels,col=ellipse.col,
-                                      hide=hide,omit=omit,omit.col=omit.col)
-    }
+    fill <- set.ellipse.colours(ns=ns,levels=levels,col=ellipse.fill,
+                                hide=hide,omit=omit,omit.col=omit.fill)
+    stroke <- set.ellipse.colours(ns=ns,levels=levels,col=ellipse.stroke,
+                                  hide=hide,omit=omit,omit.col=omit.stroke)
     if (show.ellipses==0){ # points and or text
         if (missing(cex)) cex <- 1
-        if (missing(bg)) bg <- colour
-        plot_points(xy[,'X'],xy[,'Y'],bg=bg,cex=cex,
+        if (missing(bg)) bg <- fill
+        plot_points(xy[,'X'],xy[,'Y'],bg=bg,cex=cex,col=stroke,
                     show.numbers=show.numbers,hide=hide,omit=omit,...)
     } else if (show.ellipses==1){ # error ellipse
         if (missing(cex)) cex <- 0.25
@@ -148,7 +161,7 @@ scatterplot <- function(xy,xlim=NA,ylim=NA,alpha=0.05,
             if (!any(is.na(xy[i,]))){
                 covmat <- cor2cov2(xy[i,'sX'],xy[i,'sY'],xy[i,'rXY'])
                 ell <- ellipse(xy[i,'X'],xy[i,'Y'],covmat,alpha=alpha)
-                graphics::polygon(ell,col=colour[i])
+                graphics::polygon(ell,col=fill[i],border=stroke[i])
                 if (show.numbers) graphics::text(xy[i,'X'],xy[i,'Y'],i)
                 else graphics::points(xy[i,'X'],xy[i,'Y'],pch=19,cex=cex)
             }
@@ -157,24 +170,25 @@ scatterplot <- function(xy,xlim=NA,ylim=NA,alpha=0.05,
         if (missing(cex)) cex <- 0.5
         if (show.numbers)
             graphics::text(xy[plotit,'X'],xy[plotit,'Y'],
-                           sn[plotit],adj=c(0,1),col=colour)
+                           sn[plotit],adj=c(0,1),col=stroke)
         else
             graphics::points(xy[plotit,'X'],xy[plotit,'Y'],
-                             pch=19,cex=cex,col=colour)
+                             pch=21,cex=cex,col=stroke,bg=fill)
         fact <- stats::qnorm(1-alpha/2)
         dx <- fact*xy[plotit,'sX']
         dy <- fact*xy[plotit,'sY']
         graphics::arrows(xy[plotit,'X'],xy[plotit,'Y']-dy,
                          xy[plotit,'X'],xy[plotit,'Y']+dy,
                          code=3,angle=90,
-                         length=0.05,col=colour)
+                         length=0.05,col=stroke)
         graphics::arrows(xy[plotit,'X']-dx,xy[plotit,'Y'],
                          xy[plotit,'X']+dx,xy[plotit,'Y'],
                          code=3,angle=90,
-                         length=0.05,col=colour)
+                         length=0.05,col=stroke)
     }
     if (!nolevels & addcolourbar){
-        colourbar(z=levels[calcit],col=ellipse.col,clabel=clabel)
+        colourbar(z=levels[calcit],fill=ellipse.fill,
+                  stroke=ellipse.stroke,clabel=clabel)
     }
 }
 
